#define esdAna_cxx
// The class definition in esdAna.h has been generated automatically
// by the ROOT utility TTree::MakeSelector(). This class is derived
// from the ROOT class TSelector. For more information on the TSelector
// framework see $ROOTSYS/README/README.SELECTOR or the ROOT User Manual.

// The following methods are defined in this file:
//    Begin():        called everytime a loop on the tree starts,
//                    a convenient place to create your histograms.
//    SlaveBegin():   called after Begin(), when on PROOF called only on the
//                    slave servers.
//    Process():      called for each event, in this function you decide what
//                    to read and fill your histograms.
//    SlaveTerminate: called at the end of the loop on the tree, when on PROOF
//                    called only on the slave servers.
//    Terminate():    called at the end of the loop on the tree,
//                    a convenient place to draw/fit your histograms.
//
// To use this file, try the following session on your Tree T:
//
// Root > T->Process("esdAna.C")
// Root > T->Process("esdAna.C","some options")
// Root > T->Process("esdAna.C+")
//
//Origin : Marc Dujardin, Yves Schutz CERN

#include "esdAna.h"
#include <TCanvas.h>
#include <TLegend.h>
#include <TVector3.h>
#include <TFileMerger.h> 
#include <Riostream.h>

//____________________________________________________________________________________
esdAna::esdAna(TTree *) :
 TSelector(),
 fChain(0),
 fESD(0), 
 fCurrentEvent(0),
 hEMCALPos(0),
 hPHOSPos(0),
 hEMCALEnergy(0),
 hPHOSEnergy(0),
 hEMCALDigits(0),
 hPHOSDigits(0),
 hEMCALRecParticules(0),
 hPHOSRecParticules(0),
 hPHOSPhotons(0),
 hEMCALPhotons(0),
 hEMCALInvariantMass(0),
 hPHOSInvariantMass(0),
 hEMCALDigitsEvent(0),
 hPHOSDigitsEvent(0)
 {
  // Constructor. Initialization of pointers

}

//____________________________________________________________________________________
esdAna::~esdAna() {
  // Remove all pointers

  delete fESD;

  // histograms are in the output list and deleted when the output
  // list is deleted by the TSelector dtor
}

//____________________________________________________________________________________
void esdAna::Begin(TTree *)
{
   // The Begin() function is called at the start of the query.
   // When running with PROOF Begin() is only called on the client.
   // The tree argument is deprecated (on PROOF 0 is passed).

   TString option = GetOption();

}

//____________________________________________________________________________________
void esdAna::SlaveBegin(TTree *tree)
{
   // The SlaveBegin() function is called after the Begin() function.
   // When running with PROOF SlaveBegin() is called on each slave server.
   // The tree argument is deprecated (on PROOF 0 is passed).

   Init(tree);

   TString option = GetOption();

   // create histograms on each slave server

   hEMCALPos           = new TNtuple("EMCALPos"        , "Position in EMCAL" , "x:y:z");
   hPHOSPos            = new TNtuple("PHOSPos"         , "Position in PHOS"  , "x:y:z");
   hPHOSEnergy         = new TH1F("PHOSEnergy"         , "PHOSEnergy"        , 400, 0., 50. ) ;
   hEMCALEnergy        = new TH1F("EMCALEnergy"        , "EMCALEnergy"       , 400, 0., 50. ) ;
   hEMCALDigits        = new TH1I("EMCALDigitsCluster" , "EMCALDigits"       , 20 , 0 , 20  ) ;
   hPHOSDigits         = new TH1I("PHOSDigitsCluster"  , "PHOSDigits"        , 20 , 0 , 20  ) ;
   hEMCALRecParticules = new TH1F("EMCALRecParticules" , "EMCALRecParticules", 20 , 0., 20. ) ;
   hPHOSRecParticules  = new TH1F("PHOSRecParticules"  , "PHOSRecParticules" , 20 , 0., 20. ) ;
   hPHOSPhotons        = new TH1I("PHOSPhotons"        , "PHOSPhotons"       , 20 , 0 , 20  ) ;
   hEMCALPhotons       = new TH1I("EMCALPhotons"       , "EMCALPhotons"      , 20 , 0 , 20  ) ;
   hEMCALInvariantMass = new TH1F("EMCALInvariantMass" , "EMCALInvariantMass", 200, 0., 200.) ;
   hPHOSInvariantMass  = new TH1F("PHOSInvariantMass"  , "PHOSInvariantMass" , 200, 0., 200.) ;
   hEMCALDigitsEvent   = new TH1I("EMCALDigitsEvent"   , "EMCALDigitsEvent"  , 30 , 0 , 30  ) ;
   hPHOSDigitsEvent    = new TH1I("PHOSDigitsEvent"    , "PHOSDigitsEvent"   , 30 , 0 , 30  ) ;
 
}

//____________________________________________________________________________________
void esdAna::Init(TTree *tree)
{
   // The Init() function is called when the selector needs to initialize
   // a new tree or chain. Typically here the branch addresses of the tree
   // will be set. It is normaly not necessary to make changes to the
   // generated code, but the routine can be extended by the user if needed.
   // Init() will be called many times when running with PROOF.

   // Set branch addresses
   if ( !tree ) 
     return ;
   fChain = tree ;
   fChain->SetBranchAddress("ESD", &fESD) ;

}

//____________________________________________________________________________________
Bool_t esdAna::Notify()
{
   // The Notify() function is called when a new file is opened. This
   // can be either for a new TTree in a TChain or when when a new TTree
   // is started when using PROOF. Typically here the branch pointers
   // will be retrieved. It is normaly not necessary to make changes
   // to the generated code, but the routine can be extended by the
   // user if needed.

   // Get branch pointers

  fChain->SetCacheSize(0);
  return kTRUE;
}

//____________________________________________________________________________________
Bool_t esdAna::Process(Long64_t entry)
{
  // The Process() function is called for each entry in the tree (or possibly
  // keyed object in the case of PROOF) to be processed. The entry argument
  // specifies which entry in the currently loaded tree is to be processed.
  // It can be passed to either TTree::GetEntry() or TBranch::GetEntry()
  // to read either all or the required parts of the data. When processing
  // keyed objects with PROOF, the object is already loaded and is available
  // via the fObject pointer.
  //
  // This function should contain the "body" of the analysis. It can contain
  // simple or elaborate selection criteria, run algorithms on the data
  // of the event and typically fill histograms.
    
  // Reads one event
  GetEntry(entry) ;
  fCurrentEvent += 1 ; 
  if ( !((fCurrentEvent-1)%100) ) {
    printf("%s ----> ", (dynamic_cast<TChain *>(fChain))->GetFile()->GetName()) ;  
    printf("Processing event # %d\n", fCurrentEvent) ; 
  }
  //************************  EMCAL *************************************
  Int_t              firstEmcalCluster       = fESD->GetFirstEMCALCluster() ;
  const Int_t numberOfEmcalClusters   = fESD->GetNumberOfEMCALClusters() ;

  TVector3 ** emcalVector = new TVector3*[numberOfEmcalClusters] ;
  Float_t *  emcalPhotonsEnergy = new Float_t[numberOfEmcalClusters] ;

  Int_t      emcalCluster ; 
  Int_t      numberOfEmcalClustersv1 = 0 ; 
  Int_t      numberOfPhotonsInEmcal  = 0 ;
  Int_t      numberOfDigitsInEmcal   = 0 ;  

  // loop over all the EMCAL Cluster
  for(emcalCluster = firstEmcalCluster ; emcalCluster < firstEmcalCluster + numberOfEmcalClusters ; emcalCluster++) {
    AliESDCaloCluster * caloCluster = fESD->GetCaloCluster(emcalCluster) ;
    if (caloCluster) {
      Float_t pos[3] ;
      hEMCALPos->Fill(pos[0],pos[1],pos[2]) ;
      if(caloCluster->GetClusterType() == AliESDCaloCluster::kClusterv1) {  
	caloCluster->GetGlobalPosition(pos) ;
	hEMCALEnergy->Fill(caloCluster->GetClusterEnergy()) ;
	hEMCALDigits->Fill(caloCluster->GetNumberOfDigits()) ;
	numberOfEmcalClustersv1++ ;
	numberOfDigitsInEmcal += caloCluster->GetNumberOfDigits() ;    
	// Float_t * pid = clus->GetPid() ;
	// if(pid[AliPID::kPhoton]>0.9){
	emcalVector[numberOfPhotonsInEmcal] = new TVector3(pos[0],pos[1],pos[2]) ;
	emcalPhotonsEnergy[numberOfPhotonsInEmcal] = caloCluster->GetClusterEnergy() ;
	numberOfPhotonsInEmcal++ ;
      }
    }
  } // EMCAL clusters loop

  hEMCALRecParticules->Fill(numberOfEmcalClustersv1);
  hEMCALPhotons->Fill(numberOfPhotonsInEmcal);
  hEMCALDigitsEvent->Fill(numberOfDigitsInEmcal);
    
  // invariant Mass
  if (numberOfPhotonsInEmcal > 1 ) {
    Int_t emcalPhoton1, emcalPhoton2 ; 
    for(emcalPhoton1 = 0 ; emcalPhoton1 < numberOfPhotonsInEmcal ; emcalPhoton1++) {
      for(emcalPhoton2 = emcalPhoton1 + 1 ; emcalPhoton2 < numberOfPhotonsInEmcal ; emcalPhoton2++) {
	Float_t tempMass = TMath::Sqrt( 2 * emcalPhotonsEnergy[emcalPhoton1] * emcalPhotonsEnergy[emcalPhoton2] * 
					( 1 - TMath::Cos( emcalVector[emcalPhoton1]->Angle(*emcalVector[emcalPhoton2])) 
					  )
					);
	hEMCALInvariantMass->Fill(tempMass*1000.);
      }
    }    
  }
  

  //************************  PHOS *************************************
  Int_t              firstPhosCluster       = fESD->GetFirstPHOSCluster() ;
  const Int_t numberOfPhosClusters   = fESD->GetNumberOfPHOSClusters() ;

  TVector3 ** phosVector = new TVector3*[numberOfPhosClusters] ;

  Float_t *  phosPhotonsEnergy = new Float_t[numberOfPhosClusters] ;
  Int_t      phosCluster ; 
  Int_t      numberOfPhotonsInPhos  = 0 ;
  Int_t      numberOfDigitsInPhos   = 0 ;
  
  // loop over the PHOS Cluster
  for(phosCluster = firstPhosCluster ; phosCluster < firstPhosCluster + numberOfPhosClusters ; phosCluster++) {
    AliESDCaloCluster * caloCluster = fESD->GetCaloCluster(phosCluster) ;
    if (caloCluster) {
      Float_t pos[3] ;
      caloCluster->GetGlobalPosition( pos ) ;
      hPHOSEnergy->Fill( caloCluster->GetClusterEnergy() ) ;
      //hPHOSPos->Fill( pos[0], pos[1], pos[2] ) ;
      hPHOSDigits->Fill( caloCluster->GetNumberOfDigits() ) ;
      numberOfDigitsInPhos += caloCluster->GetNumberOfDigits() ;
      Float_t * pid = caloCluster->GetPid() ;
      if(pid[AliPID::kPhoton] > 0.9) {
	phosVector[numberOfPhotonsInPhos] = new TVector3(pos[0],pos[1],pos[2]) ;
	phosPhotonsEnergy[numberOfPhotonsInPhos]=caloCluster->GetClusterEnergy() ;
	numberOfPhotonsInPhos++;
      }
    }
  } //PHOS clusters
    
  hPHOSRecParticules->Fill(numberOfPhosClusters);
  hPHOSPhotons->Fill(numberOfPhotonsInPhos);
  hPHOSDigitsEvent->Fill(numberOfDigitsInPhos);

  // invariant Mass
  if (numberOfPhotonsInPhos > 1 ) {
    Int_t phosPhoton1, phosPhoton2 ; 
    for(phosPhoton1 = 0 ; phosPhoton1 < numberOfPhotonsInPhos ; phosPhoton1++) {
      for(phosPhoton2 = phosPhoton1 + 1 ; phosPhoton2 < numberOfPhotonsInPhos ; phosPhoton2++) {      
	Float_t tempMass = TMath::Sqrt( 2 * phosPhotonsEnergy[phosPhoton1] * phosPhotonsEnergy[phosPhoton2] *
					( 1 - TMath::Cos(phosVector[phosPhoton1]->Angle(*phosVector[phosPhoton2])) ) 
					);
	hPHOSInvariantMass->Fill(tempMass*1000.);
      }
    }    
  }

  delete [] phosVector ; 
  delete [] emcalVector ; 

  return kTRUE;
}

//____________________________________________________________________________________
void esdAna::SlaveTerminate()
{
  // The SlaveTerminate() function is called after all entries or objects
  // have been processed. When running with PROOF SlaveTerminate() is called
  // on each slave server.
  
  // Add the histograms to the output on each slave server

}

//____________________________________________________________________________________
void esdAna::Terminate()
{
   // The Terminate() function is the last function to be called during
   // a query. It always runs on the client, it can be used to present
   // the results graphically or save the results to file.
  
  //cout << "enter Terminate" << endl ; 
  TFile * file = TFile::Open("esdAna.root", "RECREATE");
  hEMCALPos->Write();
  hPHOSPos->Write();
  //cout << "DEBUGG 1" << endl ; 
  hEMCALEnergy->Write();
  hPHOSEnergy->Write();
  hEMCALDigits->Write();
  hPHOSDigits->Write();
  hEMCALRecParticules->Write();
  //cout << "DEBUGG 3" << endl ; 
  hPHOSRecParticules->Write();
  hPHOSPhotons->Write();
  hEMCALPhotons->Write();
  hEMCALInvariantMass->Write();
  //cout << "DEBUGG 4" << endl ; 
  hPHOSInvariantMass->Write();
  hEMCALDigitsEvent->Write();
  hPHOSDigitsEvent->Write();
  printf("EMCALEnergy Mean        : %5.3f , RMS : %5.3f \n", hEMCALEnergy->GetMean(),        hEMCALEnergy->GetRMS()        ) ;
  printf("PHOSEnergy Mean         : %5.3f , RMS : %5.3f \n", hPHOSEnergy->GetMean(),         hPHOSEnergy->GetRMS()         ) ;
  printf("EMCALDigits Mean        : %5.3f , RMS : %5.3f \n", hEMCALDigits->GetMean(),        hEMCALDigits->GetRMS()        ) ;
  printf("PHOSDigits Mean         : %5.3f , RMS : %5.3f \n", hPHOSDigits->GetMean(),         hPHOSDigits->GetRMS()         ) ;
  printf("EMCALRecParticules Mean : %5.3f , RMS : %5.3f \n", hEMCALRecParticules->GetMean(), hEMCALRecParticules->GetRMS() ) ;
  printf("PHOSRecParticules Mean  : %5.3f , RMS : %5.3f \n", hPHOSRecParticules->GetMean(),  hPHOSRecParticules->GetRMS()  ) ;
  printf("PHOSPhotons Mean        : %5.3f , RMS : %5.3f \n", hPHOSPhotons->GetMean(),        hPHOSPhotons->GetRMS()        ) ;
  printf("EMCALPhotons Mean       : %5.3f , RMS : %5.3f \n", hEMCALPhotons->GetMean(),       hEMCALPhotons->GetRMS()       ) ;
  printf("EMCALInvariantMass Mean : %5.3f , RMS : %5.3f \n", hEMCALInvariantMass->GetMean(), hEMCALInvariantMass->GetRMS() ) ;
  printf("PHOSInvariantMass Mean  : %5.3f , RMS : %5.3f \n", hPHOSInvariantMass->GetMean(),  hPHOSInvariantMass->GetRMS()  ) ;
  printf("EMCALDigitsEvent Mean   : %5.3f , RMS : %5.3f \n", hEMCALDigitsEvent->GetMean(),   hEMCALDigitsEvent->GetRMS()   ) ;
  printf("PHOSDigitsEvent Mean    : %5.3f , RMS : %5.3f \n", hPHOSDigitsEvent->GetMean(),    hPHOSDigitsEvent->GetRMS()    ) ;

  file->Close() ;


  TCanvas  * c1 = new TCanvas("Energy", "Energy", 1);
  c1->SetLogy();
  hEMCALEnergy->SetAxisRange(0, 25.);
  hEMCALEnergy->SetTitle("p+p #sqrt{14 GeV}");
  hEMCALEnergy->SetXTitle("Energy (GeV)");
  hEMCALEnergy->Draw();
  hPHOSEnergy->SetAxisRange(0, 25.);
  hPHOSEnergy->SetLineColor(2);
  hPHOSEnergy->Draw("SAME");
  TLegend  * l1 = new TLegend(0.6, 0.7, 0.75, 0.85);
  l1->AddEntry(hEMCALEnergy,"EMCAL","l");
  l1->AddEntry(hPHOSEnergy,"PHOS","l");
  l1->Draw();
  c1->Update();

  TCanvas  * c2 = new TCanvas("DigitsCluster", "DigitsCluster", 1);
  hEMCALDigits->SetAxisRange(0, 25.);
  hEMCALDigits->SetTitle("p+p #sqrt{14 GeV}");
  hEMCALDigits->SetXTitle("DigitsPerCluster");
  hEMCALDigits->Draw();
  hPHOSDigits->SetAxisRange(0,25.);
  hPHOSDigits->SetLineColor(2);
  hPHOSDigits->Draw("SAME");
  TLegend * l2 = new TLegend(0.6, 0.7, 0.75, 0.85);
  l2->AddEntry(hEMCALDigits, "EMCAL", "l");
  l2->AddEntry(hPHOSDigits,"PHOS", "l");
  l2->Draw() ; 
  c2->Update();

  TCanvas  * c3 = new TCanvas("RecParticules", "RecParticules", 1);
  c3->SetLogy();
  hEMCALRecParticules->SetAxisRange(0, 25.);
  hEMCALRecParticules->SetTitle("p+p #sqrt{14 GeV}");
  hEMCALRecParticules->SetXTitle("RecParticules");
  hEMCALRecParticules->Draw();
  hPHOSRecParticules->SetAxisRange(0, 25.);
  hPHOSRecParticules->SetLineColor(2);
  hPHOSRecParticules->Draw("SAME");
  TLegend * l3 = new TLegend(0.6, 0.7, 0.75, 0.85);
  l3->AddEntry(hEMCALRecParticules, "EMCAL", "l");
  l3->AddEntry(hPHOSRecParticules, "PHOS", "l");
  l3->Draw();
  c3->Update();

  TCanvas  * c4 = new TCanvas("Photons", "Photons", 1);
  c4->SetLogy();
  hEMCALPhotons->SetAxisRange(0, 25.);
  hEMCALPhotons->SetTitle("p+p #sqrt{14 GeV}");
  hEMCALPhotons->SetXTitle("Photons");
  hEMCALPhotons->Draw();
  hPHOSPhotons->SetAxisRange(0,25.);
  hPHOSPhotons->SetLineColor(2);
  hPHOSPhotons->Draw("SAME");
  TLegend * l4 = new TLegend(0.6, 0.7, 0.75, 0.85);
  l4->AddEntry(hEMCALPhotons, "EMCAL","l");
  l4->AddEntry(hPHOSPhotons, "PHOS","l");
  l4->Draw();
  c4->Update();

  TCanvas  * c5 = new TCanvas("InvariantMass", "InvariantMasse",1);
  hEMCALInvariantMass->SetTitle("p+p #sqrt{14 GeV}");
  hEMCALInvariantMass->SetXTitle("InvariantMass (MeV/c²)");
  hEMCALInvariantMass->Draw();
  hPHOSInvariantMass->SetLineColor(2);
  hPHOSInvariantMass->Draw("SAME");
  TLegend * l5 = new TLegend(0.2, 0.7, 0.45, 0.85);
  l5->AddEntry(hEMCALInvariantMass, "EMCAL", "l");
  l5->AddEntry(hPHOSInvariantMass, "PHOS", "l");
  l5->Draw();
  c5->Update();

  TCanvas  * c6 = new TCanvas("DigitsEvent", "DigitsEvent", 1);
  TLegend * l6 = new TLegend(0.6, 0.7, 0.75, 0.85);
  c6->SetLogy();
  hEMCALDigitsEvent->SetAxisRange(0, 40.);
  hEMCALDigitsEvent->SetTitle("p+p #sqrt{14 GeV}");
  hEMCALDigitsEvent->SetXTitle("DigitsPerEvent");
  hEMCALDigitsEvent->Draw();
  hPHOSDigitsEvent->SetAxisRange(0,40.);
  hPHOSDigitsEvent->SetLineColor(2);
  hPHOSDigitsEvent->Draw("SAME");
  l6->AddEntry(hEMCALDigitsEvent,"EMCAL","l");
  l6->AddEntry(hPHOSDigitsEvent, "PHOS","l");
  l6->Draw();
  c6->Update();

  c1->Print("Energy.eps");
  c2->Print("DigitsPerCluster.eps");
  c3->Print("RecParticules.eps");
  c4->Print("Photons.eps");
  c5->Print("InvariantMass.eps");
  c6->Print("DigitsPerEvent.eps");

  gROOT->ProcessLine(".!tar -zcvf Graph.tar.gz *.eps esdAna.root");
 
  printf("!!! All the eps files are in Graph.tar.gz !!! \n");

}

